'use strict';

var mode = 'online';
var isCavasInit = false;
var effects = [];
var currentEffect = null;
var canvas = document.getElementById('canvas'),
  context = canvas.getContext('2d');
var htmlNode = document.getElementsByTagName('html')[0];
var encoder = new GIFEncoder();
var string = 'asdfasdfasdfasdfasdf asdfasdfasdfasdfasdfasdf ';

function initCanvasAndRun(effectType) {
  setTimeout(function() {
    if (htmlNode.offsetHeight === 0 || htmlNode.offsetWidth === 0) {
      initCanvasAndRun(effectType);
      return;
    }
    canvas.width = htmlNode.offsetWidth * 2;
    canvas.style.width = htmlNode.offsetWidth + 'px';
    if (mode === 'dev') {
      canvas.height = htmlNode.offsetHeight;
      canvas.style.height = htmlNode.offsetHeight / 2 + 'px';
    } else {
      canvas.height = htmlNode.offsetHeight * 2;
      canvas.style.height = htmlNode.offsetHeight + 'px';
    }
    isCavasInit = true;
    runEffect(effectType);
  }, 200);
}

function initCanvasAndSend(effectType) {
  setTimeout(function() {
    canvas.width = htmlNode.offsetWidth;
    canvas.style.width = canvas.width + 'px';
    if (mode === 'dev') {
      canvas.height = htmlNode.offsetHeight / 2;
      canvas.style.height = htmlNode.offsetHeight / 2 + 'px';
    } else {
      canvas.height = htmlNode.offsetHeight;
      canvas.style.height = htmlNode.offsetHeight + 'px';
    }
    isCavasInit = true;
    sendImage(effectType);
  }, 200);
}

function getScript(effectType) {
  var script = document.createElement("script");
  script.src = './effects/' + effectType + '/effect.js';
  script.type = "text/javascript";
  document.getElementsByTagName("html")[0].appendChild(script);
  return script;
}

function runEffect(effectType) {
  if (!isCavasInit) {
    initCanvasAndRun(effectType);
    return;
  }
  canvas.width = htmlNode.offsetWidth * 2;
  canvas.style.width = htmlNode.offsetWidth + 'px';
  if (mode === 'dev') {
    canvas.height = htmlNode.offsetHeight;
    canvas.style.height = htmlNode.offsetHeight / 2 + 'px';
  } else {
    canvas.height = htmlNode.offsetHeight * 2;
    canvas.style.height = htmlNode.offsetHeight + 'px';
  }
  if (currentEffect) {
    currentEffect.stop();
  }
  var hasEffect = false;
  effects.forEach(function(effect) {
    if (effect.name === effectType) {
      hasEffect = true;
      currentEffect = effect;
    }
  });
  var param = {
    canvas: canvas,
    encoder: encoder,
    inputString: mode === 'dev' ? string : getInputString(effectType)
  };
  if (hasEffect === true) {
    currentEffect.preview(param);
  } else {
    var script = getScript(effectType);
    script.onload = function() {
      setTimeout(function() {
        effects.forEach(function(effect) {
          if (effect.name === effectType) {
            hasEffect = true;
            currentEffect = effect;
          }
        });
        if (hasEffect === true) {
          if (currentEffect.hasProgress && window.BTJsHandler && window.BTJsHandler.hasProgress) {
            window.BTJsHandler.hasProgress(currentEffect.name, true);
          }
          currentEffect.preview(param);
        } else {
          console.log('run effect failed');
        }
      }, 10);
    };
  }
}

function sendImage(effectType) {
  if (!isCavasInit) {
    initCanvasAndSend(effectType);
    return;
  }
  canvas.width = htmlNode.offsetWidth;
  canvas.style.width = canvas.width + 'px';
  if (mode === 'dev') {
    canvas.height = htmlNode.offsetHeight / 2;
    canvas.style.height = htmlNode.offsetHeight / 2 + 'px';
  } else {
    canvas.height = htmlNode.offsetHeight;
    canvas.style.height = htmlNode.offsetHeight + 'px';
  }
  if (currentEffect) {
    currentEffect.stop();
  }
  var hasEffect = false;
  effects.forEach(function(effect) {
    if (effect.name === effectType) {
      hasEffect = true;
      currentEffect = effect;
    }
  });
  var param = {
    canvas: canvas,
    encoder: encoder,
    inputString: mode === 'dev' ? string : getInputString(effectType)
  };
  if (hasEffect === true) {
    currentEffect.send(param);
    if (mode === 'dev') {
      setImageData();
    } else {
      generateAndSend();
    }
  } else {
    var script = getScript(effectType);
    script.onload = function() {
      setTimeout(function() {
        effects.forEach(function(effect) {
          if (effect.name === effectType) {
            hasEffect = true;
            currentEffect = effect;
          }
        });
        if (hasEffect === true) {
          currentEffect.send(param);
          if (window.needsCanceled) {
            return;
          }
          if (mode === 'dev') {
            setImageData();
          } else {
            generateAndSend();
          }
        } else {
          console.log('send Image failed');
        }
      }, 10);
    };
  }
}

function getInputString(effectType) {
  return window.BTJsHandler.getProcessString(effectType);
}

function generateAndSend() {
  window.BTJsHandler.postGIF(encode64(encoder.stream().getData()));
}
window.generateAndSend = generateAndSend;
window.runEffect = runEffect;
window.sendImage = sendImage;

window.devSend = function(type) {
  sendImage(type);
};

function setImageData() {
  document.getElementById('result').src = 'data:image/gif;base64,' + encode64(encoder.stream().getData());
}